@@ -55,11 +55,14 @@ import select
 from gettext import gettext as _
 from gettext import bindtextdomain, textdomain
 
-from debian_bundle import debian_support
+try:
+    from debian import debian_support
+except ImportError:
+    from debian_bundle import debian_support
 
 def safe_tar_extract(filename, destination):
     """Safely extracts a tarball into destination"""
-    logging.debug('safe_tar_extract: %s to %s', (filename, destination))
+    logging.debug('safe_tar_extract: %s to %s' % (filename, destination))
 
     rfd = tarfile.open(filename)
     dangerous_file = False
@@ -197,7 +200,7 @@ class Backend(dbus.service.Object):
                 'org.freedesktop.PolicyKit1.Authority')
         try:
             # we don't need is_challenge return here, since we call with AllowUserInteraction
-            (is_auth, unused, details) = self.polkit.CheckAuthorization(
+            (is_auth, unused, details) = self.polkit.CheckAuthorization( # pylint: disable=unused-variable
                     ('unix-process', {'pid': dbus.UInt32(pid, variant_level=1),
                         'start-time': dbus.UInt64(0, variant_level=1)}),
                     privilege, {'': ''}, dbus.UInt32(1), '', timeout=600)
@@ -256,7 +259,7 @@ class Backend(dbus.service.Object):
                                  universal_newlines=True)
         output = command.communicate()
         ret = command.wait()
-        if ret is not 0:
+        if ret != 0:
             os.rmdir(mntdir)
             if ret == 32:
                 try:
@@ -278,7 +281,7 @@ class Backend(dbus.service.Object):
 
         if os.path.exists(mnt):
             ret = subprocess.call(['umount', mnt])
-            if ret is not 0:
+            if ret != 0:
                 logging.warning(" _unmount_drive: Error unmounting %s" % mnt)
             try:
                 os.rmdir(mnt)
@@ -315,7 +318,7 @@ class Backend(dbus.service.Object):
                     os.makedirs(dest)
                 call = subprocess.Popen(['dpkg-repack', 'dell-recovery'],
                                         cwd=dest, universal_newlines=True)
-                (out, err) = call.communicate()
+                (out, err) = call.communicate() # pylint: disable=unused-variable
         else:
             logging.debug("_test_for_new_dell_recovery: RP Distro %s doesn't match our distro %s, not injecting updated package", rp_distro, package_distro)
 
@@ -478,7 +481,7 @@ class Backend(dbus.service.Object):
                     new_name += '.zip'
                 elif os.path.exists(fishie) and tarfile.is_tarfile(fishie):
                     new_name += '.tgz'
-                shutil.copy_file(fishie, os.path.join(dest, new_name))
+                shutil.copyfile(fishie, os.path.join(dest, new_name))
 
         #If dell-recovery needs to be injected into the image
         if dell_recovery_package:
@@ -490,7 +493,7 @@ class Backend(dbus.service.Object):
                 logging.debug("Repacking dell-recovery using dpkg-repack")
                 call = subprocess.Popen(['dpkg-repack', 'dell-recovery'],
                                         cwd=dest, universal_newlines=True)
-                (out, err) = call.communicate()
+                (out, err) = call.communicate() # pylint: disable=unused-variable
             else:
                 logging.debug("Adding manually included dell-recovery package, %s", dell_recovery_package)
                 shutil.copy(dell_recovery_package, dest)
@@ -557,8 +560,8 @@ class Backend(dbus.service.Object):
             #RHEL disks have .discinfo
             elif os.path.exists(os.path.join(mntdir, '.discinfo')):
                 with open(os.path.join(mntdir, '.discinfo'), 'r') as rfd:
-                    timestamp = rfd.readline().strip('\n')
-                    distributor_string = rfd.readline().strip('\n')
+                    timestamp = rfd.readline().strip('\n') # pylint: disable=unused-variable
+                    distributor_string = rfd.readline().strip('\n') # pylint: disable=unused-variable
                     arch = rfd.readline().strip('\n')
                 distributor = "redhat"
                 distributor_str += ' ' + arch
@@ -592,7 +595,7 @@ arch %s, distributor_str %s, bto_platform %s" % (bto_version, distributor, relea
             chain1 = subprocess.Popen(cmd1, stdin=chain0.stdout, stdout=subprocess.PIPE)
             chain2 = subprocess.Popen(cmd2, stdin=chain1.stdout, stdout=subprocess.PIPE,
                                       universal_newlines=True)
-            out, err = chain2.communicate()
+            out, err = chain2.communicate() # pylint: disable=unused-variable
             if chain2.returncode is None:
                 chain2.wait()
             if out:
@@ -629,7 +632,7 @@ arch %s, distributor_str %s, bto_platform %s" % (bto_version, distributor, relea
                         version = out[0]
                         date = out[1]
                 else:
-                    version = test_initrd(['isoinfo', '-J', '-i', recovery, '-x', '/casper/initrd.lz'])
+                    version = test_initrd(['isoinfo', '-J', '-i', recovery, '-x', '/casper/initrd'])
 
         else:
             mntdir = self.request_mount(recovery, "r", sender, conn)
@@ -644,8 +647,8 @@ arch %s, distributor_str %s, bto_platform %s" % (bto_version, distributor, relea
                     version = rfd.readline().strip('\n')
                     date = rfd.readline().strip('\n')
             #no /bto.xml or /bto_version found, check initrd for bootsrap files
-            elif os.path.exists(os.path.join(mntdir, 'casper', 'initrd.lz')):
-                version = test_initrd(['cat', os.path.join(mntdir, 'casper', 'initrd.lz')])
+            elif os.path.exists(os.path.join(mntdir, 'casper', 'initrd')):
+                version = test_initrd(['cat', os.path.join(mntdir, 'casper', 'initrd')])
 
         return (version, date, platform)
 
@@ -710,7 +713,7 @@ arch %s, distributor_str %s, bto_platform %s" % (bto_version, distributor, relea
             #Search for a flat file first (or a manifest for later)
             logging.debug("query_have_dell_recovery: Searching mount point %s", recovery)
             interesting_files = []
-            for root, dirs, files in os.walk(recovery, topdown=False):
+            for root, dirs, files in os.walk(recovery, topdown=False):  # pylint: disable=unused-variable
                 for fname in files:
                     if 'dell-recovery' in fname and (fname.endswith('.deb') or fname.endswith('.rpm')):
                         logging.debug("query_have_dell_recovery: Found in %s", os.path.join(root, fname))
@@ -764,7 +767,7 @@ arch %s, distributor_str %s, bto_platform %s" % (bto_version, distributor, relea
             rfd.extract(prepackage, tmpdir)
             rfd.close()
             self.xml_obj.load_bto_xml(os.path.join(tmpdir, 'prepackage.dell'))
-            our_os = lsb_release.get_lsb_information()['RELEASE']
+            our_os = lsb_release.get_os_release()['RELEASE']
             package_os = self.xml_obj.fetch_node_contents('os')
             if our_os != package_os:
                 valid = 0
@@ -830,17 +833,17 @@ arch %s, distributor_str %s, bto_platform %s" % (bto_version, distributor, relea
                         env['PATH'] = path
 
         ret = subprocess.call(['/usr/sbin/update-grub'], env=env)
-        if ret is not 0:
+        if ret != 0:
             raise RestoreFailed("error updating grub configuration")
 
         ret = subprocess.call(['/usr/sbin/grub-reboot', entry])
-        if ret is not 0:
+        if ret != 0:
             raise RestoreFailed("error setting one time grub entry")
 
         if reboot:
             logging.debug("Prepare to reboot")
             ret = subprocess.call(["/sbin/reboot", "--force"])
-            if ret is not 0:
+            if ret != 0:
                 raise RestoreFailed("error invoking reboot")
 
 
@@ -938,14 +941,25 @@ arch %s, distributor_str %s, bto_platform %s" % (bto_version, distributor, relea
             xorrisoargs.append('-boot-info-table')
             xorrisoargs.append('-isohybrid-mbr')
             xorrisoargs.append(os.path.join('/', 'usr', 'lib', 'ISOLINUX', 'isohdpfx.bin'))
+        else:
+            raise CreateFailed("Unable to locate isolinux to build hybrid MBR")
 
         #include bootloader as eltorito if we have it
-        if os.path.exists(os.path.join(mntdir, 'boot', 'efi.img')):
+        if os.path.exists(os.path.join(mntdir, 'boot', 'grub', 'efi.img')):
+            xorrisoargs.append('-eltorito-alt-boot')
+            xorrisoargs.append('-e')
+            xorrisoargs.append('boot/grub/efi.img')
+            xorrisoargs.append('-no-emul-boot')
+            xorrisoargs.append('-isohybrid-gpt-basdat')
+        elif os.path.exists(os.path.join(mntdir, 'boot', 'efi.img')):
             xorrisoargs.append('-eltorito-alt-boot')
             xorrisoargs.append('-e')
             xorrisoargs.append('boot/efi.img')
             xorrisoargs.append('-no-emul-boot')
             xorrisoargs.append('-isohybrid-gpt-basdat')
+        else:
+            raise CreateFailed("Unable to locate EFI bootloader image")
+
 
         #disable 32 bit bootloader if it was there.
         grub_path = os.path.join(mntdir, 'boot', 'grub', 'i386-pc')
@@ -965,9 +979,6 @@ arch %s, distributor_str %s, bto_platform %s" % (bto_version, distributor, relea
             xorrisoargs.append('efi.factory')
             shutil.copytree(os.path.join(mntdir, 'efi.factory'), os.path.join(tmpdir, 'efi'))
 
-        #if no bootstrap in RP, we'll put it in the initrd
-        bootstrap_initrd = not os.path.exists(os.path.join(mntdir, 'scripts', 'bootstrap.sh'))
-
         #Renerate UUID
         os.mkdir(os.path.join(tmpdir, '.disk'))
         os.mkdir(os.path.join(tmpdir, 'casper'))
@@ -976,9 +987,7 @@ arch %s, distributor_str %s, bto_platform %s" % (bto_version, distributor, relea
          old_uuid) = create_new_uuid(os.path.join(mntdir, 'casper'),
                         os.path.join(mntdir, '.disk'),
                         os.path.join(tmpdir, 'casper'),
-                        os.path.join(tmpdir, '.disk'),
-                        new_compression="auto",
-                        include_bootstrap=bootstrap_initrd)
+                        os.path.join(tmpdir, '.disk'))
         self.stop_progress_thread()
         xorrisoargs.append('-m')
         xorrisoargs.append(os.path.join('.disk', old_uuid))
@@ -1035,13 +1044,13 @@ arch %s, distributor_str %s, bto_platform %s" % (bto_version, distributor, relea
             if not os.path.exists(os.path.join(mntdir, 'boot', 'grub', 'dejavu-sans-12.pf2')):
                 ret = subprocess.call(['grub-mkfont', '/usr/share/fonts/truetype/ttf-dejavu/DejaVuSans.ttf',
                                        '-s=12', '--output=%s' % os.path.join(tmpdir, 'boot', 'grub', 'dejavu-sans-12.pf2')])
-                if ret is not 0:
+                if ret != 0:
                     raise CreateFailed("Creating GRUB fonts failed.")
 
             if not os.path.exists(os.path.join(mntdir, 'boot', 'grub', 'dejavu-sans-bold-14.pf2')):
                 ret = subprocess.call(['grub-mkfont', '/usr/share/fonts/truetype/ttf-dejavu/DejaVuSans-Bold.ttf',
                                        '-s=14', '--output=%s' % os.path.join(tmpdir, 'boot', 'grub', 'dejavu-sans-bold-14.pf2')])
-                if ret is not 0:
+                if ret != 0:
                     raise CreateFailed("Creating GRUB fonts failed.")
 
         #if we previously backed up a grub.cfg or common.cfg
@@ -1069,18 +1078,25 @@ arch %s, distributor_str %s, bto_platform %s" % (bto_version, distributor, relea
         xorrisoargs.append(mntdir + '/')
 
         #ISO Creation
-        seg1 = subprocess.Popen(xorrisoargs,
-                              stderr=subprocess.PIPE,
-                              stdout=subprocess.PIPE,
-                              universal_newlines=True)
-        pipe = seg1.stderr
+        try:
+            seg1 = subprocess.Popen(xorrisoargs,
+                                  stderr=subprocess.PIPE,
+                                  stdout=subprocess.PIPE,
+                                  universal_newlines=True)
+            pipe = seg1.stderr
 
-        fcntl.fcntl(
-            pipe.fileno(),
-            fcntl.F_SETFL,
-            fcntl.fcntl(pipe.fileno(), fcntl.F_GETFL) | os.O_NONBLOCK)
+            fcntl.fcntl(
+                pipe.fileno(),
+                fcntl.F_SETFL,
+                fcntl.fcntl(pipe.fileno(), fcntl.F_GETFL) | os.O_NONBLOCK)
+
+            retval = seg1.poll()
+        except OSError as e:
+            if isinstance(e, FileNotFoundError):
+                raise CreateFailed("xorriso is not installed")
+            else:
+                raise e
 
-        retval = seg1.poll()
         logging.debug(" create_ubuntu: xorriso debug")
         while (retval is None):
             readx = select.select([pipe.fileno()], [], [])[0]
@@ -1094,7 +1110,7 @@ arch %s, distributor_str %s, bto_platform %s" % (bto_version, distributor, relea
                         if (progress[-1:] == '%'):
                             self.report_progress(_('Building ISO'), progress[:-1])
             retval = seg1.poll()
-        if retval is not 0:
+        if retval != 0:
             logging.error(" create_ubuntu: xorriso exited with a nonstandard return value.")
             logging.error("  cmd: %s" % xorrisoargs)
             logging.error("  stderror: %s" % pipe.readlines())
