@@ -39,6 +39,7 @@ import logging
 import hashlib
 import io
 import locale
+import uuid
 
 ##                ##
 ##Common Variables##
@@ -154,7 +155,7 @@ def check_vendor():
     """Checks to make sure that the app is running on Dell HW"""
     path = '/sys/class/dmi/id/'
     variables = ['bios_vendor', 'sys_vendor']
-    valid = [b'dell', b'alienware', b'wyse', b'innotek']
+    valid = [b'dell', b'alienware', b'wyse', b'qemu']
     for var in variables:
         target = os.path.join(path, var)
         if os.path.exists(target):
@@ -239,6 +240,7 @@ def fetch_output(cmd, data='', environment=os.environ):
     try:
         (out, err) = proc.communicate(data)
     except UnicodeDecodeError as error:
+        logging.warning ("unicode decode error: %s" % error)
         if locale.getpreferredencoding() != "UTF-8":
             os.environ["LC_ALL"] = "C.UTF-8"
             locale.setlocale(locale.LC_ALL, '')
@@ -480,104 +482,96 @@ def walk_cleanup(directory):
         os.rmdir(directory)
 
 def create_new_uuid(old_initrd_directory, old_casper_directory,
-                    new_initrd_directory, new_casper_directory,
-                    new_compression="auto",
-                    include_bootstrap=False):
+                    new_initrd_directory, new_casper_directory):
     """ Regenerates the UUID contained in a casper initramfs
-        Supported compression types:
-        * auto (auto detects lzma/gzip)
-        * lzma
-        * gzip
-        * None
         Returns full path of the old initrd and casper files (for blacklisting)
     """
     tmpdir = tempfile.mkdtemp()
 
     #Detect the old initramfs stuff
     try:
-        old_initrd_files = glob.glob('%s/initrd*' % old_initrd_directory)
+        old_initrd_file = glob.glob('%s/initrd*' % old_initrd_directory)[0]
     except Exception as msg:
         logging.warning("create_new_uuid: %s" % str(msg))
         raise dbus.DBusException("Missing initrd in image.")
     try:
         old_uuid_file   = glob.glob('%s/casper-uuid*' % old_casper_directory)[0]
     except Exception as msg:
-        logging.warning("create_new_uuid: Old casper UUID not found, assuming 'casper-uuid'")
+        logging.warning("create_new_uuid: Old casper UUID not found, assuming 'casper-uuid': %s" % msg)
         old_uuid_file   = '%s/casper-uuid' % old_casper_directory
 
-    old_initrd_file = ''
-    old_compression = ''
-    old_suffix = ''
-    for fname in old_initrd_files:
-        if len(fname.split('.')) > 1:
-            old_suffix = fname.split('.')[-1]
-        if old_suffix == "lz":
-            old_compression = "lzma"
-            old_initrd_file = fname
-            break
-        elif old_suffix == "gz":
-            old_compression = "gzip"
-            old_initrd_file = fname
-            break
-        else:
-            old_suffix = ''
-
-    if not old_suffix or not old_initrd_file or not old_uuid_file:
+    if not old_initrd_file or not old_uuid_file:
         raise dbus.DBusException("Unable to detect valid initrd.")
 
-    logging.debug("create_new_uuid: old initrd %s, old uuid %s, old suffix %s, \
-old compression method %s" % (old_initrd_file, old_uuid_file,
-                              old_suffix, old_compression))
+    logging.debug("create_new_uuid: old initrd %s, old uuid %s" %
+                 (old_initrd_file, old_uuid_file))
 
-    #Extract old initramfs
-    chain0 = subprocess.Popen([old_compression, '-cd', old_initrd_file, '-S',
-                               old_suffix], stdout=subprocess.PIPE)
-    chain1 = subprocess.Popen(['cpio', '-id'], stdin=chain0.stdout, cwd=tmpdir)
-    chain1.communicate()
+    #Extract old initramfs with the new format
+    chain0 = subprocess.Popen(["/usr/bin/unmkinitramfs", old_initrd_file, "."],
+                            stdout=subprocess.PIPE, cwd=tmpdir)
+    chain0.communicate()
 
     #Generate new UUID
     new_uuid_file = os.path.join(new_casper_directory,
                                  os.path.basename(old_uuid_file))
     logging.debug("create_new_uuid: new uuid file: %s" % new_uuid_file)
-    chain0 = subprocess.Popen(['uuidgen', '-r'], stdout=subprocess.PIPE)
-    new_uuid = chain0.communicate()[0]
-    logging.debug("create_new_uuid: new UUID: %s" % new_uuid.strip())
-    for item in [new_uuid_file, os.path.join(tmpdir, 'conf', 'uuid.conf')]:
-        with open(item, "wb") as uuid_fd:
-            uuid_fd.write(new_uuid)
+    new_uuid = str(uuid.uuid4())
+    logging.debug("create_new_uuid: new UUID: %s" % new_uuid)
+    initramfs_root = os.path.join(tmpdir, 'main')
+    if not os.path.exists(initramfs_root):
+        initramfs_root = tmpdir
+    for item in [new_uuid_file, os.path.join(initramfs_root, 'conf', 'uuid.conf')]:
+        with open(item, "w") as uuid_fd:
+            uuid_fd.write("%s\n" % new_uuid)
 
-    #Newer (Ubuntu 11.04+) images may support including the bootstrap in initrd
-    if include_bootstrap:
-        chain0 = subprocess.Popen(['/usr/share/dell/casper/hooks/dell-bootstrap'], env={'DESTDIR': tmpdir, 'INJECT': '1'})
-        chain0.communicate()
+    #Add bootstrap to initrd
+    chain0 = subprocess.Popen(['/usr/share/dell/casper/hooks/dell-bootstrap'], env={'DESTDIR': initramfs_root, 'INJECT': '1'})
+    chain0.communicate()
 
     #Detect compression
-    new_suffix = ''
+    lines = ''
+    root = os.path.join(tmpdir, 'main', 'conf', 'initramfs.conf')
+    with open(root, 'r') as rfd:
+        lines = rfd.readlines()
+    new_compression = ''
+    for line in lines:
+        if line.startswith('COMPRESS='):
+            components = line.split('=')
+            if len(components) > 1:
+                new_compression = components[1].strip()
+
     if new_compression == "gzip":
-        new_suffix = '.gz'
-    elif new_compression == 'lzma':
-        new_suffix = '.lz'
-    elif new_compression == "auto":
-        new_compression = old_compression
-        new_suffix = '.' + old_suffix
-    logging.debug("create_new_uuid: new suffix: %s" % new_suffix)
-    logging.debug("create_new_uuid: new compression method: %s" % new_compression)
+        compress_command = ["gzip", "-n"]
+    elif new_compression == 'lzma' or new_compression == "xz":
+        compress_command = ["xz", "--check=crc32"]
+    elif new_compression == "lz4":
+        compress_command = ["lz4", "-9", "-l"]
+    logging.debug("create_new_uuid: compression detected: %s" % new_compression)
+    logging.debug("create_new_uuid: compression command: %s" % compress_command)
 
     #Generate new initramfs
-    new_initrd_file = os.path.join(new_initrd_directory, 'initrd' + new_suffix)
+    new_initrd_file = os.path.join(new_initrd_directory, 'initrd')
     logging.debug("create_new_uuid: new initrd file: %s" % new_initrd_file)
-    chain0 = subprocess.Popen(['find'], cwd=tmpdir, stdout=subprocess.PIPE)
-    chain1 = subprocess.Popen(['cpio', '--quiet', '-o', '-H', 'newc'],
-                               cwd=tmpdir, stdin=chain0.stdout,
-                               stdout=subprocess.PIPE)
-    with open(new_initrd_file, 'wb') as initrd_fd:
-        if new_compression:
-            chain2 = subprocess.Popen([new_compression, '-9c'],
-                                      stdin=chain1.stdout,
-                                      stdout=subprocess.PIPE)
-            initrd_fd.write(chain2.communicate()[0])
-        else:
-            initrd_fd.write(chain1.communicate()[0])
+
+    # make the early and late sections separately
+    for component in ['early', 'early2', 'main']:
+        root = os.path.join(tmpdir, component)
+        if not os.path.exists (root):
+            continue
+        chain0 = subprocess.Popen(['find'], cwd=root,
+                                stdout=subprocess.PIPE)
+        chain1 = subprocess.Popen(['cpio', '--quiet', '-o', '-H', 'newc'],
+                                cwd=root, stdin=chain0.stdout,
+                                stdout=subprocess.PIPE)
+        with open(new_initrd_file, 'ab') as initrd_fd:
+            if component == 'main':
+                chain2 = subprocess.Popen(compress_command,
+                                        stdin=chain1.stdout,
+                                        stdout=subprocess.PIPE)
+                initrd_fd.write(chain2.communicate()[0])
+            else:
+                initrd_fd.write(chain1.communicate()[0])
+
     walk_cleanup(tmpdir)
 
     return (old_initrd_file, old_uuid_file)
@@ -616,20 +610,19 @@ def write_seed(seed, keys):
 
 def mark_upgrades():
     '''Mark packages that can upgrade to upgrade during install'''
-    from apt.cache import Cache
-    cache = Cache()
-    to_install = []
-    for key in cache.keys():
-        if cache[key].is_upgradable:
-            to_install.append(key)
-    del cache
-    return to_install
 
-def mark_unconditional_debs(add_directory=''):
-    '''Finds any debs from debs/main that we want unconditionally installed
-       (but ONLY the latest version on the media)'''
+
+def mark_packages(recovery_partition):
+    '''Finds packages to install:
+        * any debs from debs/main that we want unconditionally installed
+          (but ONLY the latest version on the media)
+        * upgrades
+        * dell-recovery - if recovery partition
+        * dell-eula - if it exists
+    '''
     import apt_inst
     import apt_pkg
+    from apt.cache import Cache
 
     def parse(fname):
         """ read a deb """
@@ -644,7 +637,7 @@ def mark_unconditional_debs(add_directory=''):
     #process debs/main
     to_install = []
     my_arch = fetch_output(['dpkg', '--print-architecture']).strip()
-    for top in [ISO_MOUNT, CDROM_MOUNT, add_directory]:
+    for top in [ISO_MOUNT, CDROM_MOUNT]:
         repo = os.path.join(top, 'debs', 'main')
         if os.path.isdir(repo):
             for fname in os.listdir(repo):
@@ -653,11 +646,25 @@ def mark_unconditional_debs(add_directory=''):
                     if not modaliases and (arch == "all" or arch == my_arch):
                         to_install.append(package)
 
+    #mark upgrades and dell-recovery/dell-eula
+    cache = Cache()
+    for key in cache.keys():
+        if cache[key].is_upgradable:
+            to_install.append(key)
+            continue
+        #only install if present on the media
+        if key == 'dell-eula' and recovery_partition:
+            to_install.append(key)
+    del cache
+
+    #only install if using recovery partition
+    if recovery_partition:
+        to_install.append('dell-recovery')
+
     return to_install
 
 def create_grub_entries(target_dir='/target', rec_type='hdd'):
     '''Create GRUB entry for dell-recovery during ubiquity installation'''
-    env = os.environ
     rpart = find_factory_partition_stats()
     target_grub = '%s/etc/grub.d/99_dell_recovery' % target_dir
     #create the grub entry only when recovery partition exists
