@@ -43,6 +43,7 @@ import tarfile
 import gi
 gi.require_version('UDisks', '2.0')
 from gi.repository import GLib, UDisks
+import hashlib
 
 NAME = 'dell-bootstrap'
 BEFORE = 'language'
@@ -357,71 +358,34 @@ class Page(Plugin):
         except Exception:
             pass
 
-    def test_swap(self):
-        """Tests whether to do a swap fixup"""
-        import lsb_release
-        release = lsb_release.get_distro_information()
-
-        #starting with 17.04, we replace the whole swap partition to swap file
-        if float(release["RELEASE"]) >= 17.04:
-            try:
-                self.db.set('partman-swapfile/percentage', '50')
-                self.db.set('partman-swapfile/size', self.mem * 2048)
-            except debconf.DebconfError as err:
-                self.log(str(err))
-            return True
-
-        if (self.mem >= 32 or self.disk_size <= 64):
-            return True
-        else:
-            return False
-
-    def clean_recipe(self):
-        """Cleans up the recipe to remove swap if we have a small drive"""
-
-        if self.test_swap():
-            self.log("Performing swap recipe fixup (hdd: %i, mem: %f)" % \
-                                        (self.disk_size, self.mem))
-            try:
-                recipe = self.db.get('partman-auto/expert_recipe')
-                self.db.set('partman-auto/expert_recipe',
-                                     ' . '.join(recipe.split('.')[0:-2])+' .')
-            except debconf.DebconfError as err:
-                self.log(str(err))
-
     def remove_extra_partitions(self):
         """Removes partitions we are installing on for the process to start"""
-       #check for small disks.
-       #on small disks or big mem, don't look for extended or delete swap.
-        swap_part = EFI_SWAP_PARTITION
+        #check for small disks.
+        #on small disks or big mem, don't look for extended or delete swap.
         os_part = EFI_OS_PARTITION
 
-       # check dual boot or not
+        # check dual boot or not
         try:
             if self.db.get('dell-recovery/dual_boot') == 'true':
            ##dual boot get the partition number of OS and swap
                 os_label = self.db.get('dell-recovery/os_partition')
-                os_part,swap_part = self.dual_partition_num(os_label)
+                os_part = self.dual_partition_num(os_label)
         except debconf.DebconfError as err:
             self.log(str(err))
 
-        if self.test_swap():
-            swap_part = ''
-       #remove extras
-        for number in (os_part,swap_part):
-            if number.isdigit():
-                remove = misc.execute_root('parted', '-s', self.device, 'rm', number)
+        #remove extras
+        if os_part.isdigit():
+                remove = misc.execute_root('parted', '-s', self.device, 'rm', os_part)
                 if remove is False:
-                    self.log("Error removing partition number: %s on %s (this may be normal)'" % (number, self.device))
-                refresh = misc.execute_root('partx', '-d', '--nr', number, self.device)
+                    self.log("Error removing partition number: %s on %s (this may be normal)'" % (os_part, self.device))
+                refresh = misc.execute_root('partx', '-d', '--nr', os_part, self.device)
                 if refresh is False:
-                    self.log("Error updating partition %s for kernel device %s (this may be normal)'" % (number, self.device))
+                    self.log("Error updating partition %s for kernel device %s (this may be normal)'" % (os_part, self.device))
 
     def dual_partition_num(self,label):
        #remove UBUNTU patition for dual boot
        ##OS num
         os_part = ''
-        swap_part = ''
         digits = re.compile('\d+')
         try:
             os_path = magic.fetch_output(['readlink','/dev/disk/by-label/'+label]).split('\n')
@@ -432,13 +396,7 @@ class Page(Plugin):
                 self.log('os_path command is executed failed, the error is %s'%str(err))
         os_part = digits.search(os_path[0].split('/')[-1]).group()
 
-        with misc.raised_privileges():
-            partitions = magic.fetch_output(['parted','-s',self.device,'print']).split('\n')
-            for line in partitions:
-                if 'linux-swap' in line:
-                    swap_part = line.split()[0]
-
-        return os_part,swap_part
+        return os_part
 
     def explode_sdr(self):
         '''Explodes all content explicitly defined in an SDR
@@ -797,6 +755,23 @@ class Page(Plugin):
             self.log("rec_type %s, stage %d, device %s" % (rec_type, self.stage, self.device))
             if (rec_type == 'factory' and self.stage == 2) or rec_type == 'hdd':
                 self.fixup_factory_devices(rec_part)
+            if rec_type == 'hdd':
+                # copy old mok key so that user don't need to enroll it again.
+                rootfs = mount[0:-1] + EFI_OS_PARTITION
+                self.log("old rootfs from %s" % rootfs)
+                try:
+                    misc.execute_root('mount', '-o', 'ro', rootfs, '/mnt')
+                except:
+                    self.log("mouting old rootfs failed, give up old mok.")
+                if os.path.exists('/mnt/var/lib/shim-signed/mok/MOK.priv') and os.path.exists('/mnt/var/lib/shim-signed/mok/MOK.der'):
+                    with misc.raised_privileges():
+                        shutil.copy('/mnt/var/lib/shim-signed/mok/MOK.der', '/tmp')
+                        shutil.copy('/mnt/var/lib/shim-signed/mok/MOK.priv', '/tmp')
+                        with open('/tmp/MOK.der','rb') as f:
+                            self.log("/tmp/MOK.der %s" % hashlib.md5(f.read()).hexdigest())
+                        with open('/tmp/MOK.priv','rb') as f:
+                            self.log("/tmp/MOK.priv %s" % hashlib.md5(f.read()).hexdigest())
+                misc.execute_root('umount', '/mnt')
         except Exception as err:
             self.handle_exception(err)
             self.cancel_handler()
@@ -879,7 +854,6 @@ class Page(Plugin):
                     self.ui.toggle_progress()
                 self.sleep_network()
                 self.delete_swap()
-                self.clean_recipe()
                 self.remove_extra_partitions()
                 self.explode_sdr()
         except Exception as err:
@@ -1056,8 +1030,7 @@ manually to proceed.")
         #Install grub
         self.status("Installing GRUB", 88)
         ##If we don't have grub binaries, build them
-        grub_files = ['/cdrom/efi/boot/bootx64.efi',
-                      '/cdrom/efi/boot/grubx64.efi']
+        grub_files = ['bootx64.efi', 'grubx64.efi']
 
         ##Mount ESP
         mount = misc.execute_root('mount', self.device + esp_part, '/mnt/efi')
@@ -1070,10 +1043,11 @@ manually to proceed.")
             os.makedirs(direct_path)
 
             #copy boot loader files
-            for item in grub_files:
-                if not os.path.exists(item):
-                    raise RuntimeError("Error, %s doesn't exist." % item)
-                shutil.copy(item, direct_path)
+            for root, dirs, files in os.walk('/cdrom', topdown=False):
+                for bootloader in grub_files:
+                    for f in files:
+                        if bootloader in f.lower():
+                            shutil.copy(os.path.join(root,f), direct_path)
 
             #find old entries
             bootmgr_output = magic.fetch_output(['efibootmgr', '-v']).split('\n')
@@ -1274,21 +1248,15 @@ class Install(InstallPlugin):
         shutil.copy(pool_cmd, os.path.join(self.target, 'tmp', os.path.basename(pool_cmd)))
         install_misc.chrex(self.target, os.path.join('/tmp', os.path.basename(pool_cmd)))
 
-        #Stuff that is installed on all configs without fish scripts
-        to_install += magic.mark_unconditional_debs()
-
         #install dell-recovery only if there is an RP
         if rec_part:
             #hide the recovery partition as default
             try:
                 recovery = magic.find_factory_partition_stats()
                 command = ('parted', '-a', 'optimal', '-s', recovery['slave'], 'set', str(recovery['number']), 'msftres', 'on' )
-                misc.execute_root(*command)                
+                misc.execute_root(*command)
             except Exception:
                 pass
-            
-            to_install.append('dell-recovery')
-            to_install.append('dell-eula')
 
             #block os-prober in grub-installer
             os.rename('/usr/bin/os-prober', '/usr/bin/os-prober.real')
@@ -1324,7 +1292,8 @@ class Install(InstallPlugin):
             with open(fname, 'w') as wfd:
                 wfd.write('WARRANTY=%s\n' % destination)
 
-        to_install += magic.mark_upgrades()
+        #mark all upgrades and unconditional installs
+        to_install += magic.mark_packages(rec_part)
 
         self.remove_ricoh_mmc()
 
